/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.users.ImportUsersDialog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("net.jazz.ajax.ui.internal.Dialog");

dojo.require("com.ibm.team.repository.web.client.internal.userRegistry");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

(function() {

var DELAY = 500;
var RUNNING_SEARCH_MSG = "Running search...";
var FETCH_ERR_MSG = "Error running query";
var KEY_ESCAPE = 27;
var KEY_RETURN = 13;
var KEY_ENTER = 14;
var KEY_SPACE = 32;

var UserRegistryClient = com.ibm.team.repository.web.client.internal.userRegistry;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

function addOption(selectList, option) {
	if (dojo.isIE) {
		var newOption = new Option(option.text, option.value);
		newOption.emailAddress = option.emailAddress;
		var index = selectList.options.length;
		for (var i=0; i < selectList.options.length; i++) {
			if (selectList.options[i].text > option.text) {
				index = i;
				break;
			}
		}
		for (var i = selectList.options.length; i > index; i--) {
			selectList.options[i] = selectList.options[i-1];
		}
		selectList[index] = newOption;
	}
	else {
		option.selected = false;
		var inserted = false;
		for (var i=0; i < selectList.options.length; i++) {
			if (selectList.options[i].text > option.text) {
				selectList.insertBefore(option, selectList.options[i]);
				inserted = true;
				break;
			}
		}
		if (!inserted) {
			selectList.appendChild(option);
		}
	}
}

function removeOption(selectList, option) {
	if (dojo.isIE) {
		var index = selectList.options.length;
		for (var i=0; i < selectList.options.length; i++) {
			if (selectList.options[i] === option) {
				index = i;
				break;
			}
		}
		for (; index < selectList.options.length - 1; index++) {
			selectList.options[index] = selectList.options[index + 1];
		}
		selectList.options.length = selectList.options.length - 1;
	}
	else {
		try {
			selectList.removeChild(option);
		}
		catch (err) {
			// don't throw an error if the option wasn't in the list to begin with
		}
	}
}
	
dojo.declare("com.ibm.team.repository.web.ui.internal.admin.users.ImportUsersDialog", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/templates/ImportUsersDialog.html"),
	
	constructor: function (args) {
		if (args == null) {
			args = new Object();
		}
		this.width = args.width || "450px";
		var properties = {
			"width" : "450px",
			"removeUsersOnAdd": true,
			"searchLimit" : 50
		};
		this.selectedUsersList = new Array();
	},
	
	postCreate: function () {
		this.userSelector.size = 12;
		this.userSelector.multiple = true;
		this.selectedUsers.size = 6;
		this.selectedUsers.multiple = true;
		this.dialog = new net.jazz.ajax.ui.internal.Dialog({
			contentNode: this.contentNode,
			primaryTitle: "Import Users",
			width: this.width,
			isClosable: this.isClosable
		});
		this.connect(this.searchText,"onkeypress",function (e) {
			if (e.keyCode === KEY_RETURN || e.keyCode === KEY_ENTER) {
				this._runSearch();
			}
			else if (e.keyCode === KEY_ESCAPE) {
				this.dialog.close();
			}
		});
		this.connect(this.searchButton,"onclick","_runSearch");
		this.connect(this.selectButton,"onclick","_runSelect");
		this.connect(this.deselectButton,"onclick","_runDeselect");
		this.connect(this.cancelButton,"onclick",function (e) {
			dojo.stopEvent(e);
			this.dialog.close();
		});
		this.connect(this.okButton,"onclick","_onOk");
		this.connect(this.userSelector,"ondblclick","_runSelect");
		this.connect(this.userSelector,"onkeydown","_handleKeyDown");
		this.connect(this.userSelector,"onchange","_handleSelectorChange");
		this.connect(this.selectedUsers,"onchange","_handleSelectedChange");
		this.connect(this.dialog,"onClose","onClose");
		this.searchText.focus();
	},
	
	_runSearch: function (e, query) {
		if (e != null) {
			dojo.stopEvent(e);
		}
		if (this.searchText.value != "") {
			var args = {
				searchText: query == null ? this.searchText.value : query,
				hideExistingUsers: true
			};
			this.lastQueryTerm = args.searchText;
			var srh = new ServiceResponseHandler(this, "runSearchCallback", "_queryError");
			this._setStatusMessage("Running search...");
			UserRegistryClient.searchRegistry(srh, args);
		}
	},
	
	runSearchCallback: function (data) {
		this._clearStatusMessage();
		var users = data.externalUsers;
		if (dojo.isIE) {
			this.userSelector.options.length = 0;
		}
		else {
			dojox.data.dom.removeChildren(this.userSelector);
		}
		if (users != null) {
			var selectedUserMap = {};
			for (var i=0; i < this.selectedUsers.options.length; i++) {
				selectedUserMap[this.selectedUsers.options[i].value] = true;
			}
			users = users.sort(function (a,b) {
				if (a.fullNames == null || a.fullNames.length === 0) {
					return 1;
				}
				else if (b.fullNames == null || b.fullNames.length === 0) {
					return -1;
				}
				else {
					return a.fullNames[0] === b.fullNames[0] ? 0 : a.fullNames[0] > b.fullNames[0] ? 1 : -1;
				}
			});
			for (var i=0; i < users.length; i++) {
				// filter out users that do not have a name or do not have exactly one user ID
				if (users[i].fullNames != null && users[i].fullNames.length > 0 &&
						users[i].userIds != null && users[i].userIds.length === 1) {
					if (selectedUserMap[users[i].userIds[0]]) {
						continue;
					}
					var option = new Option(users[i].fullNames[0], users[i].userIds[0]);
					if (users[i].emailAddresses != null && users[i].emailAddresses.length > 0) {
						option.emailAddress = users[i].emailAddresses[0];
					}
					if (dojo.isIE) {
						this.userSelector.options[this.userSelector.options.length] = option;
					}
					else {
						this.userSelector.appendChild(option);
					}
				}
			}
		}
		this._handleSelectedChange();
	},
	
	_runSelect: function (e) {
		dojo.stopEvent(e);
		for (var i=0; i < this.userSelector.options.length; i++) {
			if (this.userSelector.options[i].selected) {
				var option = this.userSelector.options[i]
				addOption(this.selectedUsers, option);
				removeOption(this.userSelector, option);
				i--;
			}
		}
		this._handleSelectorChange();
		this._handleSelectedChange();
	},
	
	_runDeselect: function (e) {
		dojo.stopEvent(e);
		for (var i=0; i < this.selectedUsers.options.length; i++) {
			if (this.selectedUsers.options[i].selected) {
				var option = this.selectedUsers.options[i];
				removeOption(this.selectedUsers, option);
				i--;
			}
		}
		this._runSearch(e, this.lastQueryTerm);
	},
	
	_onOk: function (e) {
		dojo.stopEvent(e);
		var users = [];
		for (var i=0; i < this.selectedUsers.options.length; i++) {
			users.push({
				name: this.selectedUsers.options[i].text,
				userId: this.selectedUsers.options[i].value,
				emailAddress: this.selectedUsers.options[i].emailAddress
			});
		}
		this.onOk(users);
	},
	
	_handleKeyDown: function (e) {
		if ((e.keyCode == KEY_RETURN || e.keyCode == KEY_ENTER) && this.selectedUsersList.length > 0) {
			// TODO: handle keys presses
		}
	},
	
	_handleSelectorChange: function (e) {
		var count = 0;
		var selected = null;
		for (var i=0; i < this.userSelector.options.length; i++) {
			if (this.userSelector.options[i].selected) {
				selected = this.userSelector.options[i];
				count++;
			}
		}
		this.selectButton.disabled = (count === 0);
		dojox.data.dom.removeChildren(this.userInfoDiv);
		if (count === 1) {
			if (selected.emailAddress != null) {
				this.userInfoDiv.appendChild(document.createTextNode(selected.emailAddress));
			}
		}
		else if (count > 1) {
			this.userInfoDiv.appendChild(document.createTextNode(count + " users selected"));
		}
	},
	
	_handleSelectedChange: function (e) {
		var selected = false;
		for (var i=0; i < this.selectedUsers.options.length; i++) {
			if (this.selectedUsers.options[i].selected) {
				selected = true;
			}
		}
		this.deselectButton.disabled = !selected;
		this.okButton.disabled = this.selectedUsers.options.length === 0;
	},
	
	onOk: function (selectedUsers) {
		this.dialog.close();
	},
	
	onClose: function () {},
	
	_queryError: function (err) {
		this._setStatusMessage(err.message, true);
	},
	
	_clearStatusMessage: function() {
		this.searchStatus.style.visibility = "hidden";
	},
	
	_setStatusMessage: function(msg, isError) {
		this.searchStatus.style.visibility = "visible";
		dojox.data.dom.replaceChildren(this.searchStatus, document.createTextNode(msg));
		this.searchStatus.className= isError ? "ErrorStatus" : "NormalStatus";
	}
});

})();